-- Filename:     seq_statements.vhd
-- Filetype:     VHDL code (behaviour)
-- Date:         15 mar 2016
-- Update:       -
-- Description:  An example of a sequential statements
-- Author:       J. op den Brouw
-- State:        demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2016, De Haagse Hogeschool
--
-- This is an example of the equential statement if and case.
--
-- We need std_logic et al.

library ieee;
use ieee.std_logic_1164.all;

-- The entity
entity seq_statements is
    port (A, B    : in std_logic;
          SEL     : in std_logic_vector (1 downto 0);
			 EN      : in std_logic;
			 CLK     : in std_logic;
			 D       : in std_logic;
			 DATA_IN : in std_logic;
			 
          R, S     : out std_logic;
          P        : out std_logic;
          F        : out std_logic_vector (2 downto 0);
			 G        : out std_logic;
			 Z        : out std_logic;
			 Q        : out std_logic;
			 DATA_OUT : out std_logic
         );
end entity seq_statements;

-- The architecture
architecture example of seq_statements is

-- Internal signals
signal Aint, Bint, Cint : std_logic;

begin

    -- This is an example of a classic if-then-elsif-else statement.
	 -- Don't forget the else part or you will wind up with latches!
    IF_EX1: process (A, B) is
    begin
      if A = '1' and B = '0' then
        P <= '0';
      elsif A = '0' and B = '1' then
        P <= '0';
      else         -- don't forget
        P <= '1';
      end if;
    end process;
	 
    -- In this example we first assign default values to the outputs
	 -- and only reassign for curtain combinations of the inputs.
	 IF_EX2: process (A,B) is
    begin
      R <= '0';    -- default value
      S <= '1';    -- default value 
      if A = '1' and B = '1' then
        R <= '1';
        S <= '0';
      end if;       -- no else!
      if A = '1' and B = '0' then
        R <= '1';
      end if;       -- no else!
    end process;

    -- This is a classic example of a case-when statement. Don't forget the
	 -- when others.
	 CASE_EX: process (SEL) is
    begin
      case SEL is
        when "00" =>   F <= "000";
        when "01" =>   F <= "110";
        when "10" =>   F <= "110";
        when "11" =>   F <= "011";
        when others => F <= "000";  -- catch-all
      end case;
    end process;

    -- This is a classic example of a case-when statement. Don't forget the
	 -- when others. Don't forget A and B in the sensitivity list
	 CASE_EX2: process (SEL, A, B) is
    begin
      case SEL is
        when "00" =>   G <= A and B;
        when "01" =>   G <= A or B;
        when "10" =>   G <= A xor B;
        when "11" =>   G <= not B;
        when others => G <= 'X';  -- catch-all
      end case;
    end process;

    -- This is a classic example of a active high level triggered latch.
    -- Process is sensitive for both EN and D. Please note that it is
    -- good practice to design clock synchronous systems and latches
    -- should (if not: must) be avoided.
    LATCH: process (EN, D) is
    begin
      if EN = '1' then
        Z <= D;
      end if;
    end process;

    -- This is a classic example of a positive edge-triggered flip-flop.
    -- Note the use of signal attribute 'event. There is not asynchrous reset.
    -- Process is only sensitive for CLK, not for D!
    DFF: process (CLK) is
    begin
      if CLK'event and CLK = '1' then
        Q <= D;
      end if;
    end process;

    -- This is an example of a four-stage shift register. All data transfers
    -- occur at the positive edge.  There is not asynchrous reset. Note the
    -- use of the standard function rising_edge()
    SHFT: process (CLK) is
    begin
      if rising_edge(CLK) then
        Aint        <= DATA_IN;
        Bint        <= Aint;
        Cint        <= Bint;
        DATA_OUT    <= Cint;
      end if;
    end process;

end architecture example;
